using System;
using System.Reflection;
using System.Threading;
using MBN.Enums;
using MBN.Exceptions;
using Microsoft.SPOT.Hardware;

namespace MBN.Modules
{
    /// <summary>
    /// MiKroBusNet NetMF Driver for a MikroE Flame Click.
    /// <para><b>The Tilt Click is a Generic Module.</b></para>
    /// <para><b>Pins used :</b> An, Int</para>
    /// <para><b>References required:</b> MikroBus.Net, Microsoft.SPOT.Native, Microsoft.Spot.Hardware, mscorlib</para>
    /// </summary>
    /// <example>Example usage:
    /// <code language = "C#">
    /// using MBN;
    /// using MBN.Modules;
    /// using Microsoft.SPOT;
    /// using System.Threading;
    ///
    /// namespace Examples
    /// {
    ///     public class Program
    ///     {
    ///         private static FlameClick flame;
    /// 
    ///         public static void Main()
    ///         {
    ///             flame = new FlameClick(Hardware.SocketOne)
    ///             flame.FlameStateChanged += OnFlameDetected;
    ///             Thread.Sleep(Timeout.Infinite);
    ///         }
    ///
    ///         private static void OnFlameDetected(object sender, FlameClick.FlameState state)
    ///         {
    ///              Debug.Print("Flame State - " + (state == FlameClick.FlameState.Absent ? " no flame detected" : "flame detected"));
    ///         }
    ///     }
    /// }
    /// </code>
    /// <code language = "VB">
    /// Option Explicit On
    /// Option Strict On
    ///
    /// Imports System.Threading
    /// Imports MBN
    /// Imports MBN.Modules
    /// Imports Microsoft.SPOT
    ///
    /// Namespace Examples
    /// 	Public Class Program
    /// 
    /// 		Private Shared flame As FlameClick
    ///
    /// 		Public Shared Sub Main()
    /// 			flame = New FlameClick(Hardware.SocketOne)
    /// 			AddHandler flame.FlameStateChanged, AddressOf FlameFlameStateChanged
    /// 			Thread.Sleep(Timeout.Infinite1)
    /// 		End Sub
    ///
    /// 		Private Shared Sub FlameFlameStateChanged(sender As Object, flame As FlameClick.FlameState)
    ///             Debug.Print("Flame Status - " + (flame == FlameClick.FlameState.Absent ? " no flame detected" : "flame detected"));
	/// 	    End Sub
    /// 	End Class
    /// End Namespace
    /// </code>
    /// </example>
    public class FlameClick : IDriver
    {
        #region Fields
        
        internal static AnalogInput _flameAnalogValue;
        internal static InterruptPort flameInterrupt;

        #endregion

        #region Public ENUM

        /// <summary>
        /// The current state of the state.
        /// </summary>
        public enum FlameState
        {
            /// <summary>
            /// No state is detected.
            /// </summary>
            Absent = 0,
            /// <summary>
            /// A state is present and detected.
            /// </summary>
            Present = 1
        }

        #endregion

        #region CTOR

        /// <summary>
        /// Initializes a new instance of the <see cref="FlameClick"/> class.
        /// </summary>
        /// <param name="socket">The <see cref="Hardware.Socket"/> that the Flame Click is plugged into.</param>
        public FlameClick(Hardware.Socket socket)
        {
            try
            {
                Hardware.CheckPins(socket, socket.An, socket.Int);

                _flameAnalogValue = new AnalogInput(socket.AnalogChannel);
                flameInterrupt = new InterruptPort(socket.Int, true, Port.ResistorMode.PullUp, Port.InterruptMode.InterruptEdgeBoth);
                flameInterrupt.OnInterrupt += OnFlameInterrupt;
            }
            catch (PinInUseException ex)
            {
                throw new PinInUseException(ex.Message);
            }
        }

        #endregion

        #region Private Methods
        
        void OnFlameInterrupt(uint data1, uint data2, DateTime time)
        {
            if (FlameStateChanged != null) FlameStateChanged(this, data2 == 1 ? FlameState.Present : FlameState.Absent);
        }

        #endregion

        #region Public Methods
        
        /// <summary>
        /// Directly reads the voltage from the photo transistor.
        /// </summary>
        /// <param name="numberOfSamples">The number of samples to read used for smoothing out voltage spikes. Defaults to 20 samples.</param>
        /// <returns>A Double in the range of 0.0 to 3.3 VDC. Zero meaning no state presence and 3.3 for full saturation.</returns>
        /// <remarks>The value returned is independent of the threshold potentiometer setting that is only used for interrupt generation.</remarks>
        /// <example>
        /// <code language = "C#">
        /// while (true) Debug.Print("Read Flame Direct - " + _flame.ReadFlameDirect(50));
        /// </code>
        /// <code language = "VB">
        /// While True
	    ///     Debug.Print("Read Flame Direct - " <![CDATA[&]]> _flame.ReadFlameDirect(50))
        /// End While
        /// </code>
        /// </example>
        public Double ReadFlameDirect(UInt16 numberOfSamples = 20)
        {
            var average = 0.0F;
            for (var i = 0; i < numberOfSamples; i++) // Read  n samples for smoothing.
            {
                var tempValue = _flameAnalogValue.ReadRaw();
                average += tempValue;
                Thread.Sleep(0);
            }
            average /= numberOfSamples;
            return 3.3F * average / 4095F;

        }

        /// <summary>
        /// Resets the module
        /// </summary>
        /// <param name="resetMode">The reset mode :
        /// <para>SOFT reset : generally by sending a software command to the chip</para><para>HARD reset : generally by activating a special chip's pin</para></param>
        /// <example>None provided as this module does not support <see cref="ResetModes"/>.
        /// </example>
        /// <exception cref="System.NotImplementedException">Thrown because this module has no reset feature.</exception>
        public bool Reset(ResetModes resetMode)
        {
            throw new NotImplementedException();
        }
        
        #endregion        

        #region Public Properties

        /// <summary>
        /// Gets or sets the power mode.
        /// </summary>
        /// <example> This sample shows how to use the PowerMode property.
        /// </example>
        /// <value>
        /// The current power mode of the module.
        /// </value>
        /// <exception cref="System.NotImplementedException">Thrown if the property is set and the module doesn't support power modes.</exception>
        /// <example>Example usage: None provided as this module does not support <see cref="PowerModes"/>.
        /// <code language = "C#">
        /// // This module does not support Power Mode selection.
        /// </code>
        /// <code language = "VB">
        /// ' This module does not support Power Mode selection.
        /// </code>
        /// </example>
        public PowerModes PowerMode
        {
            get { return PowerModes.On; }
            set
            {
                throw new NotImplementedException();
            }
        }

        /// <summary>
        /// Gets the driver version.
        /// </summary>
        /// <example> This sample shows how to use the DriverVersion property.
        /// <code language="C#">
        /// Debug.Print ("Current driver version : "+ _flameClick.DriverVersion);
        /// </code>
        /// <code language = "VB">
        /// Debug.Print ("Current driver version : " <![CDATA[&]]> _flameClick.DriverVersion)
        /// </code>
        /// </example>
        /// <value>
        /// The driver version.
        /// </value>
        public Version DriverVersion
        {
            get { return Assembly.GetExecutingAssembly().GetName().Version; }
        }

        #endregion
        
        #region Events

        /// <summary>
        ///     This event is raised when the Flame Click detects the presence of a flame.
        /// </summary>
        public event FlameDetectedEventHandler FlameStateChanged;

        /// <summary>
        ///     Represents the delegate that is used for the <see cref="FlameClick.FlameStateChanged"/> event.
        /// </summary>
        /// <param name="sender">The <see cref="FlameClick"/> that raised the event.</param>
        /// <param name="state">The <see cref="FlameState"/> of the flame.</param>
        public delegate void FlameDetectedEventHandler(object sender, FlameState state);

        #endregion

    }
}
